-- name=int_dialog_character_impl
--******************************************************************************************
function public.Init ()

  public.GRID_CORNER_W = 88;
  public.GRID_CORNER_H = 88;
  public.GRID_MID_W    = 4;
  public.GRID_MID_H    = 4;

  --private.GRID_BORDER_W = 100;
  --private.GRID_BORDER_H = 100;

  --private.GRID_MIN_W = 300;
  --private.GRID_MAX_W = 500;
  --private.GRID_MIN_H = 100;
  --private.GRID_MAX_H = 500;

  public.GRID_FIXED_W = 500;
  public.GRID_FIXED_H = 300;

  private.SKIP_POS_X     = 977;

  private.SHOW_ANIM_TIME                = 0.6;
  private.CONTINUE_MOUSE_OVER_ANIM_TIME = 0.3;
  private.SKIP_MOUSE_OVER_ANIM_TIME     = 0.3;
  private.CHANGE_ANIM_TIME              = 0.8;

  ObjSet("obj_int_dialog_character_input", {
    event_mdown = int_dialog_character.ContinueClick
  });
  ObjSet("obj_int_dialog_character_skip", {
    event_mdown = int_dialog_character.SkipClick,
    event_menter = private.SkipMouseEnterAnim,
    event_mleave = private.SkipMouseLeaveAnim
  });

  for choice_num = 1, 3 do
   ObjSet( "spr_int_dialog_character_choise_"..choice_num, {
      event_menter = function() private.ChoiceMouseEnterAnim( choice_num ) end,
      event_mleave = function() private.ChoiceMouseLeaveAnim( choice_num ) end;
    });
  end;

  MsgSubscribe( Command_Interface_UpdateTextures, public.WideScreenUpdate );

  public.WideScreenUpdate();

end;
--******************************************************************************************
function public.WideScreenUpdate()

  if ( not GetWideScreen() ) then

    ObjSet( "obj_int_dialog_character_skip", { pos_x = private.SKIP_POS_X } );
    ObjSet( "obj_int_dialog_character_choise", { pos_x = 171 } );

  else

    local offset = 0.5 * ( GetAppWidth() - 1024 );

    ObjSet( "obj_int_dialog_character_skip", { pos_x = ( private.SKIP_POS_X + offset ) } );
    ObjSet( "obj_int_dialog_character_choise", { pos_x = 0 } );

  end;

end;
--******************************************************************************************
function public.ShowAnim( event_id )

  local trg = function () int_dialog_character.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;

  if common_impl.continue_visible then ObjSet( "txt_int_dialog_character_continue", { text = 'str_int_dialog_character_continue' } ); 
  else ObjSet( "txt_int_dialog_character_continue", { text = 'str_int_dialog_character_choise_question' } ); 
  end;

  ObjAnimate( "int_dialog_character", "alp", 0, 0, trg,
  {
    0.0, 3, 0,
    tme, 3, 1
  } );

end;
--******************************************************************************************
function public.HideAnim( event_id )

  local trg = function () int_dialog_character.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;

  ObjAnimate( "int_dialog_character", "alp", 0, 0, trg,
  {
    0.0, 3, 1,
    tme, 3, 0
  } );

end;
--******************************************************************************************
function public.ChangeProcAnim( event_id )

  --local obj = "grd_int_dialog_character";--"txt_int_dialog_character";
  if common_impl.continue_visible then ObjSet( "txt_int_dialog_character_continue", { text = 'str_int_dialog_character_continue' } ); 
  else ObjSet( "txt_int_dialog_character_continue", { text = 'str_int_dialog_character_choise_question' } ); 
  end;

  local obj = "obj_int_dialog_character";--"txt_int_dialog_character";
  local trg = function () int_dialog_character.EventAnimEnd( event_id ); end;
  local alp = ObjGet( obj ).alp;
  local tme = ( 1 - alp ) * ( 0.5 * private.CHANGE_ANIM_TIME );

  ObjAnimate( obj, "alp", 0, 0, trg,
  {
    0.0, 2, alp,
    tme, 2, 1
  } );

end;
--******************************************************************************************
function public.ChangeBeginAnim( event_id )

  --local obj = "grd_int_dialog_character";--"txt_int_dialog_character";
  local obj = "obj_int_dialog_character";--"txt_int_dialog_character";
  local trg = function () int_dialog_character.EventAnimEnd( event_id ); end;
  local alp = ObjGet( obj ).alp;
  local tme = alp * ( 0.5 * private.CHANGE_ANIM_TIME );

  ObjAnimate( obj, "alp", 0, 0, trg,
  {
    0.0, 1, alp,
    tme, 1, 0
  } );

end;
--******************************************************************************************
function private.ContinueMouseEnterAnim()

  SetCursor( CURSOR_HAND );

  local obj = "txt_int_dialog_character_continue_focus";
  local alp = ObjGet( obj ).alp;
  local tme = ( 1 - alp ) * private.CONTINUE_MOUSE_OVER_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 2, alp,
    tme, 2, 1
  } );

end;
--******************************************************************************************
function private.ContinueMouseLeaveAnim()

  SetCursor( CURSOR_DEFAULT );

  local obj = "txt_int_dialog_character_continue_focus";
  local alp = ObjGet( obj ).alp;
  local tme = alp * private.CONTINUE_MOUSE_OVER_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 1, alp,
    tme, 1, 0
  } );

end;
--******************************************************************************************
function private.SkipMouseEnterAnim()

  SetCursor( CURSOR_HAND );

  local obj = "txt_int_dialog_character_skip_focus";
  local alp = ObjGet( obj ).alp;
  local tme = ( 1 - alp ) * private.SKIP_MOUSE_OVER_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 2, alp,
    tme, 2, 1
  } );

end;
--******************************************************************************************
function private.SkipMouseLeaveAnim()

  SetCursor( CURSOR_DEFAULT );

  local obj = "txt_int_dialog_character_skip_focus";
  local alp = ObjGet( obj ).alp;
  local tme = alp * private.SKIP_MOUSE_OVER_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 1, alp,
    tme, 1, 0
  } );

end;
--******************************************************************************************
function private.ChoiceMouseEnterAnim( number )

  SetCursor( CURSOR_HAND );

  local obj = "spr_int_dialog_character_choise_"..number.."_focus";
  local alp = ObjGet( obj ).alp;
  local tme = ( 1 - alp ) * private.CONTINUE_MOUSE_OVER_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 2, alp,
    tme, 2, 1
  } );

end;
--******************************************************************************************
function private.ChoiceMouseLeaveAnim( number )

  SetCursor( CURSOR_DEFAULT );

  local obj = "spr_int_dialog_character_choise_"..number.."_focus";
  local alp = ObjGet( obj ).alp;
  local tme = alp * private.CONTINUE_MOUSE_OVER_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 1, alp,
    tme, 1, 0
  } );

end;

