-- name=int_dialog_hint
--******************************************************************************************
function public.Init (pos_z)

  ObjAttach( "int_dialog_hint", interface.originhub );
  ObjSet( "int_dialog_hint", { pos_z = pos_z } );

  private.SHOW_ANIM_TIME = 0.3;
  private.BUTTON_CLOSE_MOUSE_OVER_TIME = 0.3;
  
  ObjSet("spr_int_dialog_hint", {
    event_menter = function () SetCursor( CURSOR_HAND ); end,
    event_mleave = function () SetCursor( CURSOR_DEFAULT ); end
  });
  ObjSet("obj_int_dialog_hint_teleport", {
    event_menter = function () private.ButtonCloseMouseEnterAnim();SetCursor( CURSOR_HAND ); end,
    event_mleave = function () private.ButtonCloseMouseLeaveAnim();SetCursor( CURSOR_DEFAULT ); end
  });

    local w = GetScreenSize().w;
    local h = GetScreenSize().h;
    
    if h>w then
      local t = w;
      w = h;
      h = t;
    end;
    
    local size = { "680", "636" };
    local ratio = size[1]/size[2];
    
    local sX = w/1024;
    local sY = h/768;
    
    local outSize = {};
    outSize[1] = sX * size[1];
    outSize[2] = sY * size[2];
    local outNeedSizeX = outSize[2] * ratio;
    local needScaleX = outNeedSizeX / outSize[1];
   
    --DbgTrace("int_dialog_hint_impl:"..tostring(needScaleX));
    ObjSet("int_dialog_hint_impl",{ scale_x = needScaleX });
    
end;
--******************************************************************************************
function public.Destroy()

end;
--******************************************************************************************
function private.EventAnimEnd( event_id )

  if ( event_id == "show" ) then

    --int_dialog_impl.HideAnim("hide");

  elseif ( event_id == "hide" ) then

    common_impl.show_hint.dialog = nil;

    common_impl.DeleteHintFx();

    ObjSet( "int_dialog_hint", { active = false, visible = false, input = false } );

  end;

end;
--******************************************************************************************
function public.MouseDown( dialog_room )

  cmn.GotoRoom( dialog_room ); 
  int.DialogHintHide();
  common_impl.show_hint.dialog = nil;

  PlaySfx( "assets/audio/aud_teleport", 0, 0 );

end;
--******************************************************************************************
function private.ShowAnim( event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;

  ObjAnimate( "int_dialog_hint", "alp", 0, 0, trg,
  {
    0.0, 2, 0,
    tme, 2, 1
  } );

end;
--******************************************************************************************
function private.HideAnim( event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;
  ObjAnimate( "int_dialog_hint", 8, 0, 0, trg,
  {
    0.0, 2, 1,
    tme, 2, 0
  } );

end;
--******************************************************************************************
function private.ButtonCloseMouseEnterAnim()

  SetCursor( CURSOR_HAND );

  local obj = "spr_int_dialog_hint_teleport_focus";
  if (GetPerformanceMode() == 0)then
    obj = "pfx_int_dialog_hint_teleport_focus";
  end;

  local alp = ObjGet( obj ).alp;
  local tme = ( 1 - alp ) * private.BUTTON_CLOSE_MOUSE_OVER_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 2, alp,
    tme, 2, 1
  } );

end;
--******************************************************************************************
function private.ButtonCloseMouseLeaveAnim()

  SetCursor( CURSOR_DEFAULT );

  local obj = "spr_int_dialog_hint_teleport_focus";
  if (GetPerformanceMode() == 0)then
    obj = "pfx_int_dialog_hint_teleport_focus";
  end;
  local alp = ObjGet( obj ).alp;
  local tme = alp * private.BUTTON_CLOSE_MOUSE_OVER_TIME;

  ObjAnimate( obj, "alp", 0, 0, "",
  {
    0.0, 1, alp,
    tme, 1, 0
  } );

end;
--******************************************************************************************
function public.Show( room )
  
  local dialog_room = room or ""; 
  local miniback = "miniback";  

  ObjSet( "spr_int_dialog_hint", 
  {   
  input = true, 
  visible = true, 
  res = string.format( "assets/levels/%s/%s/"..miniback, 
    ng_global.progress[ prg.current ].common.chapter, 
    dialog_room)
  } );

  ObjSet( "obj_int_dialog_hint_teleport", { event_mdown = "int_dialog_hint.MouseDown( '"..dialog_room.."' );" });
  ObjSet( "spr_int_dialog_hint",          { event_mdown = "int_dialog_hint.MouseDown( '"..dialog_room.."' );" });

  local room_name = "";
  if common.GetObjectName(dialog_room) then
    room_name = common.GetObjectName(dialog_room);
  end;
  
  if ng_global.currentprogress == "std" then
    ObjSet( "txt_int_dialog_hint_room_name", { text = "pop_"..room_name } );
  else 
    ObjSet( "txt_int_dialog_hint_room_name", { text = "pop_"..room_name.."_ce" } );
  end;

  ObjSet( "int_dialog_hint",
  {
    active = true, visible = true, input = true,
    pos_x = 512, pos_y = 384
  } );

  ObjSet( "int_dialog_hint_impl", { pos_x = 0, pos_y = 0 } );

  local event_id = "show";
  private.ShowAnim( event_id );
  
end;
--******************************************************************************************
function public.Hide()

  ObjSet( "int_dialog_hint",{ input = false } );
  local event_id = "hide";
  private.HideAnim( event_id );

end;
--******************************************************************************************