-- name=int_tutorial_impl
--******************************************************************************************
function public.Init ()

  public.GRID_CORNER_W = 100;
  public.GRID_CORNER_H = 100;
  public.GRID_MID_W    =  8;
  public.GRID_MID_H    =  8;

  public.GRID_FIXED_W  = 500;
  public.GRID_FIXED_H  = 200;
  public.GRID_BORDER_H = 100;
  public.GRID_MIN_H    = 140;

  private.SHOW_ANIM_TIME = 0.3;
  private.MOUSE_OVER_ANIM_TIME = 0.3;
  private.default_arrow = ObjGet( "spr_int_tutorial_arrow" );
  private.default_arrow.name = nil;

  ObjSet("obj_int_tutorial_buttons_left", {
    event_mdown = function () int_tutorial.MouseDown('yes'); end,
    event_menter = function () common_impl.DialogButtonMouseEnter( "int_tutorial_buttons_left" ); end,
    event_mleave = function () common_impl.DialogButtonMouseLeave( "int_tutorial_buttons_left" ); end
  });
  ObjSet("obj_int_tutorial_buttons_right", {
    event_mdown = function () int_tutorial.MouseDown('skipall'); end,
    event_menter = function () common_impl.DialogButtonMouseEnter( "int_tutorial_buttons_right" ); end,
    event_mleave = function () common_impl.DialogButtonMouseLeave( "int_tutorial_buttons_right" ); end
  });
  ObjSet("obj_int_tutorial_buttons_mid", {
    event_mdown = function () int_tutorial.MouseDown('no'); end,
    event_menter = function () common_impl.DialogButtonMouseEnter( "int_tutorial_buttons_mid" ); end,
    event_mleave = function () common_impl.DialogButtonMouseLeave( "int_tutorial_buttons_mid" ); end
  });
  ObjSet("obj_int_tutorial_transpanent", {
    event_mdown = function () common_impl.Tutorial_Click( 'empty' ); end
  });
  
end;
--******************************************************************************************
function private.TutorAnimEnd( event_id )

  int.Lock( false );

  int_tutorial.EventAnimEnd( event_id );

end;
--******************************************************************************************
function public.ShowAnim( event_id, params )

  int.Lock( true );

  local trg = function () private.TutorAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;

  if ( ObjGet( "txt_int_tutorial" ).text == "str_tutorial_page_1" ) then
    ObjSet( "txt_int_tutorial_buttons_left",  { text = "str_int_tutorial_yes" } );
  elseif ( ObjGet( "txt_int_tutorial" ).text == "str_tutorial_page_2" ) then
    ObjSet( "txt_int_tutorial_buttons_left",  { text = "str_int_tutorial_ok" } );
  end;

  ObjAnimate( "int_tutorial", "alp", 0, 0, trg,
  {
    0.0, 2, 0,
    tme, 2, 1
  } );

  local pos_x = - params.width / 2 + 19;
  local pos_y = params.height / 2 - 42;

end;
--******************************************************************************************
function public.HideAnim( event_id )

  int.Lock( true );

  local trg = function () private.TutorAnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;

  ObjAnimate( "int_tutorial", "alp", 0, 0, trg,
  {
    0.0, 2, 1,
    tme, 2, 0
  } );

end;
--******************************************************************************************
function public.ArrowHide( arrow_id )

  local arrow = string.format( "spr_int_tutorial_arrow_%i", arrow_id );
  local trg = function () ObjDelete( arrow ); end;
  local tme = private.SHOW_ANIM_TIME;
  
  ObjAttach( arrow, "int_tutorial" );
  ObjAnimate( arrow, "alp", 0, 0, trg,
  {
    0.0, 2, 1,
    tme, 2, 0
  } );

end;
--******************************************************************************************
function public.ArrowAnim( arrow )

  arrow = arrow or "spr_int_tutorial_arrow";

  local arrow_params = ObjGet( arrow );

  local beg_x = arrow_params.pos_x;
  local beg_y = arrow_params.pos_y;

  local ang = arrow_params.ang;

  local sin = math.sin( ang );
  local cos = math.cos( ang );

  local end_x = 0 * cos - 20 * sin;
  local end_y = 0 * sin + 20 * cos;

  ObjAnimate( arrow, "pos_xy", 1, 0, "",
  {
    0.0, 3, beg_x,         beg_y,
    0.7, 3, beg_x - end_x, beg_y + end_y,
    1.4, 3, beg_x,         beg_y
  } );

end;
--******************************************************************************************
function public.ArrowShow( arrow_id, arrow_params )

  local arrow = string.format( "spr_int_tutorial_arrow_%i", arrow_id );

  ObjCreate( arrow, "spr" );
  
  ObjSet( arrow, private.default_arrow );

  if ( arrow_params.target ) then

    ObjAttach( arrow, arrow_params.target );
    table.remove( arrow_params, target );

  else

    ObjAttach( arrow, "int_tutorial" );

  end;

  ObjSet( arrow, arrow_params );

  public.ArrowAnim( arrow );

  ObjAnimate( arrow, "alp", 0, 0, "",
  {
    0.0, 3, 0,
    0.3, 3, 1
  } );

end
--******************************************************************************************