-- name=ho_dollhouse
--*********************************************************************************************************************
function public.Init ()
  --------------------------------------------------------------------------------------------------
  room.Define( "ho_dollhouse" );
  game.relations[ "ho_dollhouse" ].exitroom = "rm_livingroom";
  private.room_objname = "ho_dollhouse";

  private.win_dollhouse = {};

  private.win_dollhouse.task = { "sphere", "balloon", "bow_1", "bow_2", "figure", "fish", "giraffe", "key", "lips", "man", "photo", "stone_ring" };

  common_impl.hint[ "win_dollhouse" ] =
  {
    type = "get",
    get_obj = "spr_dollhouse_win",
    room = private.room_objname
  };

  ObjSet( "obj_dollhouse_clk_fish",     { event_mdown = private.ClkFish } );  
  ObjSet( "obj_dollhouse_clk_cap",      { event_mdown = private.ClkCap } );  
  ObjSet( "obj_dollhouse_clk_drumstic", { event_mdown = private.ClkDrumstic } );  

 private.InitAnims();
 --------------------------------------------------------------------------------------------------
  local win_dollhouse_func = function ()

    if ( cmn.IsEventDone( "win_dollhouse" ) ) then

      int.InventoryItemAdd( "inv_doll", "spr_dollhouse_win", "assets/levels/level2/common_inventory2/inv_doll/inv/inv_doll_morph" );
      ObjSet( "inv_doll", { drag = false } );
      ObjDelete( "gho_livingroom_dollhouse" );
      --int.DeleteTaskMap( "rm_corridor_ext", "win_dollhouse" );

    end;

  end;
  cmn.AddSubscriber( "win_dollhouse", win_dollhouse_func );
  --------------------------------------------------------------------------------------------------
  local start_dollhouse_func = function ()

      local ho_found = ng_global.progress[ "std" ][ "win_dollhouse" ].found;

      for i = 1, #ho_found do

        local name = common.GetObjectName( ho_found[ i ].item );

        if ( name ~= "dollhouse_balloon" ) then

          ObjSet( "spr_"..name, { alp = 1 } );

        else 

          ObjSet( "anm_"..name, { alp = 1 } );

        end;

        ObjSet( "obj_"..name, { input = false } );

      end;

    if ( cmn.IsEventDone( "win_dollhouse" ) ) then

      common_impl.ProcessHoStart( "dollhouse" );

    end;

  end;
  cmn.AddSubscriber( "win_dollhouse", start_dollhouse_func, "ho_dollhouse" );
  --------------------------------------------------------------------------------------------------
--*********************************************************************************************************************
--*********************************************************************************************************************
end;
function public.Load ()

end;
--*********************************************************************************************************************
function public.PreOpen ()

  cmn.InterfaceSetVisible( true );
  int.VisitMapRoom( game.relations[ private.room_objname ].exitroom );

  if ( not cmn.IsEventDone( "opn_dollhouse" ) ) then

      cmn.SetEventDone( "opn_dollhouse" );
      cmn.CallEventHandler( "opn_dollhouse" );

  end;

  local ho_tasks = {};

  if ( not cmn.IsEventStart( "win_dollhouse" ) ) then

    cmn.InitHo( "dollhouse" );

  end;
  private.Panel();
  for i = 1, #private.win_dollhouse.task do

      interface.ItemPanelItemAdd( ho_tasks, private.win_dollhouse.task[ i ] );

  end;

  int.ItemPanelShow( ho_tasks );

  common_impl.StopMusEnv( false );

--  cmn.StopAllEnv( "audio/level4/env/aud_corridor_ext_env" );
--
--  cmn.PlayAudio( "env", "audio/level2/env/aud_corridor_ext_env", 1 );
  cmn.PlayAudio( "mus", "audio/level2/mus/aud_house_mus", 1 );

end;
--*********************************************************************************************************************
function public.Open ()

  common_impl.PassingHoStart();

end;
--*********************************************************************************************************************
function public.PreClose ()

  int.ItemPanelHide();
  cmn.StopAudio( "env", "audio/level1/env/aud_ho_theme_mus_env", 0.3 );

end;
--*********************************************************************************************************************
function public.Close ()

  common_impl.PassingHoStop();
  ObjSet( "int_inventory_impl", { input = true, alp = 1 } );
  
  cmn.StopAudio( "mus", "audio/level2/mus/aud_house_mus" );

end;
--*********************************************************************************************************************
function public.HoComplete ()

  int.Lock( true );

end;
--*********************************************************************************************************************
function private.TaskComplete( name )

  local spr = "itm_dollhouse_"..name;
  local spr_shad = "spr_dollhouse_"..name.."_shadow";

  cmn.HoItemFind( spr, name );
  common_impl.PassingHoCheck();

  ObjSet( spr, { input = false } );
  ObjSet( spr_shad, { input = false } );

  ObjAnimate( spr, "alp", 0, 0, "", { 
    0,   0, 1, 
    0.3, 0, 0
  } );

  ObjAnimate( spr_shad, "alp", 0, 0, "", { 
    0,   0, 1, 
    0.3, 0, 0
  } );

  MsgSend( Event_DialogHo_Show, {} );

end;
--*********************************************************************************************************************
function public.ItemFound( item_name )

  cmn.PlayAudio( "sfx", "audio/level2/sfx/aud_ho_dollhouse_right_click" );
  local name = common.GetObjectName( item_name );
  local item_object_name = "obj_"..name;

  local item_spr_name = "spr_"..name;

  if ( name == "dollhouse_balloon" ) then

    item_spr_name = "anm_"..name;

  end;

  ObjSet( item_object_name, { input = false } );

  ObjSet( item_spr_name, { alp = 1 } );  

    if ( interface.itempanel.countdown == 0 ) then

      MsgSend( Event_DialogHo_Show, {} );

    end;

end;
--*********************************************************************************************************************
function private.Panel ()

  ObjSet( "obj_int_taskpanel", { alp = 0 } );
  ObjSet( "txt_int_taskpanel", { alp = 0 } );
  ObjSet( "int_inventory_impl", { input = false } );

  for i = 1, #private.win_dollhouse.task do

      local obj_name = "itm_dollhouse_"..private.win_dollhouse.task[ i ];
      local sub_name = "obj_dollhouse_item";
      ObjCreate( obj_name, "spr" );
      ObjAttach( obj_name, sub_name );  
      ObjSet( obj_name, 
          { 
          --event_mdown     = function () private.TaskComplete( private.win_dollhouse.task[ i ] ); end, 
          input           = false,
          res             = "assets/levels/level2/ho_dollhouse/layers/inv/"..private.win_dollhouse.task[ i ].."_inv",
          inputrect_init  = true,
          inputrect_x     = -50,
          inputrect_y     = -50,
          inputrect_w     = 100,
          inputrect_h     = 100,
          event_dragdrop  = function () cmn.HoItemFind( "itm_dollhouse_"..private.win_dollhouse.task[ i ] ); end,
          event_menter    = function () port.ShowPopUpHO( "pop_dollhouse_"..private.win_dollhouse.task[ i ] ); end,
            --event_mleave    = interface.PopupHide,
          event_startdrag = function () cmn.HoItemOff( "itm_dollhouse_"..private.win_dollhouse.task[ i ] ); end
          });

  end;

    --ObjAttach( "spr_dollhouse_ho2" , "spr_int_itempanel" );

end;
--********************************************************************************************************************* 
function private.InitAnims ()

  ObjAnimate( "spr_dollhouse_butterfly", "ang", 1, 0, "", 
  {
    0.0, 3, 0.09,
    3.0, 3, -0.09,
    6.0, 3, 0.09
  } );

  ObjAnimate( "spr_dollhouse_beasts", "ang", 1, 0, "", 
  {
    0.0, 3, 0.28,
    2.0, 3, 0,
    4.0, 3, 0.28
  } );

end;

--********************************************************************************************************************* 
function private.ClkFish ()

  int.Lock( true );

  local func_end = function ()

    int.Lock( false );

  end;

  cmn.PlayAudio( "sfx", "audio/level2/sfx/aud_toy_ball" );  
  AnimPlay( "anm_dollhouse_fish_click", "DoAnim", func_end );

end;
--********************************************************************************************************************* 
function private.ClkCap ()

  int.Lock( true );

  local func_end = function ()

    int.Lock( false );

  end;

  cmn.PlayAudio( "sfx", "audio/level2/sfx/aud_moove_hat" );  
  AnimPlay( "anm_dollhouse_cap_click", "DoAnim", func_end );

end;
--********************************************************************************************************************* 
function private.ClkDrumstic ()

  int.Lock( true );

  local func_end = function ()

    int.Lock( false );

  end;

  cmn.PlayAudio( "sfx", "audio/level2/sfx/aud_xilophone" );  
  AnimPlay( "anm_dollhouse_drumstick_click", "DoAnim", func_end );

end;
--********************************************************************************************************************* 
